@@ -4,16 +4,19 @@ import zlib
 from django.db import models
 from django.db.models import Sum
 from django.conf import settings
+from django.utils.encoding import python_2_unicode_compatible
 from django.utils.timezone import now
 from django.utils.translation import ugettext_lazy as _
 from django.core.exceptions import ObjectDoesNotExist, PermissionDenied
 
 from oscar.apps.basket.managers import OpenBasketManager, SavedBasketManager
 from oscar.apps.offer import results
+from oscar.core.utils import get_default_currency
 from oscar.core.compat import AUTH_USER_MODEL
 from oscar.templatetags.currency_filters import currency
 
 
+@python_2_unicode_compatible
 class AbstractBasket(models.Model):
     """
     Basket object
@@ -55,6 +58,7 @@ class AbstractBasket(models.Model):
 
     class Meta:
         abstract = True
+        app_label = 'basket'
         verbose_name = _('Basket')
         verbose_name_plural = _('Baskets')
 
@@ -73,7 +77,7 @@ class AbstractBasket(models.Model):
         self._lines = None
         self.offer_applications = results.OfferApplications()
 
-    def __unicode__(self):
+    def __str__(self):
         return _(
             u"%(status)s basket (owner: %(owner)s, lines: %(num_lines)d)") \
             % {'status': self.status,
@@ -162,6 +166,11 @@ class AbstractBasket(models.Model):
         The 'options' list should contains dicts with keys 'option' and 'value'
         which link the relevant product.Option model and string value
         respectively.
+
+        Returns (line, created).
+          line: the matching basket line
+          created: whether the line was created or updated
+
         """
         if options is None:
             options = []
@@ -210,6 +219,9 @@ class AbstractBasket(models.Model):
             line.quantity += quantity
             line.save()
         self.reset_offer_applications()
+
+        # Returning the line is useful when overriding this method.
+        return line, created
     add_product.alters_data = True
     add = add_product
 
@@ -270,6 +282,10 @@ class AbstractBasket(models.Model):
         basket.date_merged = now()
         basket._lines = None
         basket.save()
+        # Ensure all vouchers are moved to the new basket
+        for voucher in basket.vouchers.all():
+            basket.vouchers.remove(voucher)
+            self.vouchers.add(voucher)
     merge.alters_data = True
 
     def freeze(self):
@@ -401,6 +417,10 @@ class AbstractBasket(models.Model):
         """
         return self.offer_applications.voucher_discounts
 
+    @property
+    def has_shipping_discounts(self):
+        return len(self.shipping_discounts) > 0
+
     @property
     def shipping_discounts(self):
         """
@@ -433,7 +453,7 @@ class AbstractBasket(models.Model):
     @property
     def num_lines(self):
         """Return number of lines"""
-        return self.lines.all().count()
+        return self.all_lines().count()
 
     @property
     def num_items(self):
@@ -529,6 +549,7 @@ class AbstractBasket(models.Model):
             return 0
 
 
+@python_2_unicode_compatible
 class AbstractLine(models.Model):
     """
     A line of a basket (product and a quantity)
@@ -547,12 +568,9 @@ class AbstractLine(models.Model):
         'catalogue.Product', related_name='basket_lines',
         verbose_name=_("Product"))
 
-    # We store the stockrecord that should be used to fulfil this line.  This
-    # shouldn't really be NULLable but we need to keep it so for backwards
-    # compatibility.
+    # We store the stockrecord that should be used to fulfil this line.
     stockrecord = models.ForeignKey(
-        'partner.StockRecord', related_name='basket_lines',
-        null=True, blank=True)
+        'partner.StockRecord', related_name='basket_lines')
 
     quantity = models.PositiveIntegerField(_('Quantity'), default=1)
 
@@ -560,7 +578,7 @@ class AbstractLine(models.Model):
     # the basket.  This allows us to tell if a product has changed price since
     # a person first added it to their basket.
     price_currency = models.CharField(
-        _("Currency"), max_length=12, default=settings.OSCAR_DEFAULT_CURRENCY)
+        _("Currency"), max_length=12, default=get_default_currency)
     price_excl_tax = models.DecimalField(
         _('Price excl. Tax'), decimal_places=2, max_digits=12,
         null=True)
@@ -579,11 +597,12 @@ class AbstractLine(models.Model):
 
     class Meta:
         abstract = True
+        app_label = 'basket'
         unique_together = ("basket", "line_reference")
         verbose_name = _('Basket line')
         verbose_name_plural = _('Basket lines')
 
-    def __unicode__(self):
+    def __str__(self):
         return _(
             u"Basket #%(basket_id)d, Product #%(product_id)d, quantity"
             u" %(quantity)d") % {'basket_id': self.basket.pk,
@@ -612,8 +631,6 @@ class AbstractLine(models.Model):
     def discount(self, discount_value, affected_quantity, incl_tax=True):
         """
         Apply a discount to this line
-
-        Note that it only makes sense to apply
         """
         if incl_tax:
             if self._discount_excl_tax > 0:
@@ -645,7 +662,7 @@ class AbstractLine(models.Model):
         """
         Return a breakdown of line prices after discounts have been applied.
 
-        Returns a list of (unit_price_incl_tx, unit_price_excl_tax, quantity)
+        Returns a list of (unit_price_incl_tax, unit_price_excl_tax, quantity)
         tuples.
         """
         if not self.is_tax_known:
@@ -712,8 +729,7 @@ class AbstractLine(models.Model):
         Return the stock/price info
         """
         if not hasattr(self, '_info'):
-            # Cache the PurchaseInfo instance (note that a strategy instance is
-            # assigned to each line by the basket in the all_lines method).
+            # Cache the PurchaseInfo instance.
             self._info = self.basket.strategy.fetch_for_product(
                 self.product, self.stockrecord)
         return self._info
@@ -829,5 +845,6 @@ class AbstractLineAttribute(models.Model):
 
     class Meta:
         abstract = True
+        app_label = 'basket'
         verbose_name = _('Line attribute')
         verbose_name_plural = _('Line attributes')
