@@ -23,12 +23,14 @@ from treebeard.mp_tree import MP_Node
 
 from oscar.core.decorators import deprecated
 from oscar.core.utils import slugify
-from oscar.core.loading import get_classes, get_model
+from oscar.core.loading import get_classes, get_model, get_class
 from oscar.models.fields import NullCharField, AutoSlugField
 
 ProductManager, BrowsableProductManager = get_classes(
     'catalogue.managers', ['ProductManager', 'BrowsableProductManager'])
 
+Selector = get_class('partner.strategy', 'Selector')
+
 
 @python_2_unicode_compatible
 class AbstractProductClass(models.Model):
@@ -480,32 +482,53 @@ class AbstractProduct(models.Model):
         pairs = [attribute.summary() for attribute in attributes]
         return ", ".join(pairs)
 
+    # The two properties below are deprecated because determining minimum
+    # price is not as trivial as it sounds considering multiple stockrecords,
+    # currencies, tax, etc.
+    # The current implementation is very naive and only works for a limited
+    # set of use cases.
+    # At the very least, we should pass in the request and
+    # user. Hence, it's best done as an extension to a Strategy class.
+    # Once that is accomplished, these properties should be removed.
+
     @property
+    @deprecated
     def min_child_price_incl_tax(self):
         """
-        Return minimum child product price including tax
+        Return minimum child product price including tax.
         """
-        return self._min_child_price('price_incl_tax')
+        return self._min_child_price('incl_tax')
 
     @property
+    @deprecated
     def min_child_price_excl_tax(self):
         """
-        Return minimum child product price excluding tax
-        """
-        return self._min_child_price('price_excl_tax')
+        Return minimum child product price excluding tax.
 
-    def _min_child_price(self, property):
+        This is a very naive approach; see the deprecation notice above. And
+        only use it for display purposes (e.g. "new Oscar shirt, prices
+        starting from $9.50").
         """
-        Return minimum child product price
+        return self._min_child_price('excl_tax')
+
+    def _min_child_price(self, prop):
         """
-        prices = []
-        for child in self.children.all():
-            if child.has_stockrecords:
-                prices.append(getattr(child.stockrecord, property))
-        if not prices:
-            return None
-        prices.sort()
-        return prices[0]
+        Return minimum child product price.
+
+        This is for visual purposes only. It ignores currencies, most of the
+        Strategy logic for selecting stockrecords, knows nothing about the
+        current user or request, etc. It's only here to ensure
+        backwards-compatibility; the previous implementation wasn't any
+        better.
+        """
+        strategy = Selector().strategy()
+
+        children_stock = strategy.select_children_stockrecords(self)
+        prices = [
+            strategy.pricing_policy(child, stockrecord)
+            for child, stockrecord in children_stock]
+        raw_prices = sorted([getattr(price, prop) for price in prices])
+        return raw_prices[0] if raw_prices else None
 
     # The properties below are based on deprecated naming conventions
 
