@@ -109,7 +109,8 @@ class OrderPlacementMixin(CheckoutSessionMixin):
         order = self.place_order(
             order_number=order_number, user=user, basket=basket,
             shipping_address=shipping_address, shipping_method=shipping_method,
-            shipping_charge=shipping_charge, order_total=order_total, **kwargs)
+            shipping_charge=shipping_charge, order_total=order_total, 
+            billing_address=billing_address, **kwargs)
         basket.submit()
         return self.handle_successful_order(order)
 
