@@ -1,5 +1,5 @@
 from django import forms
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import ugettext_lazy as _, pgettext_lazy
 
 from oscar.core.loading import get_model
 from oscar.core.compat import get_user_model
@@ -10,7 +10,8 @@ ProductAlert = get_model('customer', 'ProductAlert')
 
 class UserSearchForm(forms.Form):
     email = forms.CharField(required=False, label=_("Email"))
-    name = forms.CharField(required=False, label=_("Name"))
+    name = forms.CharField(
+        required=False, label=pgettext_lazy(u"User's name", u"Name"))
 
 
 class ProductAlertUpdateForm(forms.ModelForm):
