@@ -6,9 +6,9 @@ class ActiveOfferManager(models.Manager):
     """
     For searching/creating offers within their date range
     """
-    def get_query_set(self):
+    def get_queryset(self):
         cutoff = now()
-        return super(ActiveOfferManager, self).get_query_set().filter(
+        return super(ActiveOfferManager, self).get_queryset().filter(
             models.Q(end_datetime__gte=cutoff) | models.Q(end_datetime=None),
             start_datetime__lte=cutoff).filter(status=self.model.OPEN)
 
@@ -17,6 +17,6 @@ class BrowsableRangeManager(models.Manager):
     """
     For searching only ranges which have the "is_browsable" flag set to True.
     """
-    def get_query_set(self):
-        return super(BrowsableRangeManager, self).get_query_set().filter(
+    def get_queryset(self):
+        return super(BrowsableRangeManager, self).get_queryset().filter(
             is_public=True)
