@@ -108,42 +108,3 @@ class StockRequired(Base):
         if self.num_available > 0:
             return _("In stock (%d available)") % self.num_available
         return _("Unavailable")
-
-
-class DelegateToStockRecord(Base):
-    """
-    An availability class which delegates all calls to the
-    stockrecord itself.  This will exercise the deprecated methods on the
-    stockrecord that call "partner wrapper" classes.
-
-    This is backwards compatible with Oscar<0.6
-    """
-
-    def __init__(self, product, stockrecord=None, user=None):
-        self.product = product
-        self.stockrecord = stockrecord
-        self.user = user
-
-    @property
-    def is_available_to_buy(self):
-        if self.stockrecord is None:
-            return False
-        if not self.product.get_product_class().track_stock:
-            return True
-        return self.stockrecord.is_available_to_buy
-
-    def is_purchase_permitted(self, quantity):
-        return self.stockrecord.is_purchase_permitted(
-            self.user, quantity, self.product)
-
-    @property
-    def code(self):
-        return self.stockrecord.availability_code
-
-    @property
-    def message(self):
-        return self.stockrecord.availability
-
-    @property
-    def dispatch_date(self):
-        return self.stockrecord.dispatch_date
