@@ -17,14 +17,6 @@ class PagePromotionAdmin(admin.ModelAdmin):
     list_display = ['page_url', 'content_object', 'position']
     exclude = ['clicks']
 
-    def get_form(self, request, obj=None, **kwargs):
-        form = super(PagePromotionAdmin, self).get_form(request, obj, **kwargs)
-        # Only allow links to models within the promotions app
-        form.base_fields['content_type'].queryset \
-            = form.base_fields['content_type'].queryset\
-            .filter(app_label='promotions')
-        return form
-
 
 class KeywordPromotionAdmin(admin.ModelAdmin):
     list_display = ['keyword', 'position', 'clicks']
