@@ -1,9 +1,16 @@
+from collections import defaultdict
+
 from django import forms
 from django.forms.widgets import Input
 from django.conf import settings
 from django.utils.translation import ugettext_lazy as _
+
 from haystack.forms import FacetedSearchForm
 
+from oscar.core.loading import get_class
+
+is_solr_supported = get_class('search.features', 'is_solr_supported')
+
 
 class SearchInput(Input):
     """
@@ -15,9 +22,17 @@ class SearchInput(Input):
     input_type = 'search'
 
 
+# Build a dict of valid queries
+VALID_FACET_QUERIES = defaultdict(list)
+for facet in settings.OSCAR_SEARCH_FACETS['queries'].values():
+    field_name = "%s_exact" % facet['field']
+    queries = [t[1] for t in facet['queries']]
+    VALID_FACET_QUERIES[field_name].extend(queries)
+
+
 class SearchForm(FacetedSearchForm):
     """
-    In Haystack, the search form is used for interpretting
+    In Haystack, the search form is used for interpreting
     and sub-filtering the SQS.
     """
     # Use a tabindex of 1 so that users can hit tab on any page and it will
@@ -28,6 +43,7 @@ class SearchForm(FacetedSearchForm):
 
     # Search
     RELEVANCY = "relevancy"
+    TOP_RATED = "rating"
     NEWEST = "newest"
     PRICE_HIGH_TO_LOW = "price-desc"
     PRICE_LOW_TO_HIGH = "price-asc"
@@ -36,9 +52,10 @@ class SearchForm(FacetedSearchForm):
 
     SORT_BY_CHOICES = [
         (RELEVANCY, _("Relevancy")),
+        (TOP_RATED, _("Customer rating")),
         (NEWEST, _("Newest")),
-        (PRICE_HIGH_TO_LOW, _("Price high To low")),
-        (PRICE_LOW_TO_HIGH, _("Price low To high")),
+        (PRICE_HIGH_TO_LOW, _("Price high to low")),
+        (PRICE_LOW_TO_HIGH, _("Price low to high")),
         (TITLE_A_TO_Z, _("Title A to Z")),
         (TITLE_Z_TO_A, _("Title Z to A")),
     ]
@@ -47,6 +64,7 @@ class SearchForm(FacetedSearchForm):
     # as we assume results are returned in relevancy order in the absence of an
     # explicit sort field being passed to the search backend.
     SORT_BY_MAP = {
+        TOP_RATED: '-rating',
         NEWEST: '-date_created',
         PRICE_HIGH_TO_LOW: '-price',
         PRICE_LOW_TO_HIGH: 'price',
@@ -54,34 +72,62 @@ class SearchForm(FacetedSearchForm):
         TITLE_Z_TO_A: '-title_s',
     }
     # Non Solr backends don't support dynamic fields so we just sort on title
-    if 'solr' not in settings.HAYSTACK_CONNECTIONS['default']['ENGINE']:
+    if not is_solr_supported():
         SORT_BY_MAP[TITLE_A_TO_Z] = 'title'
         SORT_BY_MAP[TITLE_Z_TO_A] = '-title'
 
     sort_by = forms.ChoiceField(
-        choices=SORT_BY_CHOICES, widget=forms.Select(), required=False)
+        label=_("Sort by"), choices=SORT_BY_CHOICES,
+        widget=forms.Select(), required=False)
+
+    @property
+    def selected_multi_facets(self):
+        """
+        Validate and return the selected facets
+        """
+        # Process selected facets into a dict(field->[*values]) to handle
+        # multi-faceting
+        selected_multi_facets = defaultdict(list)
+
+        for facet_kv in self.selected_facets:
+            if ":" not in facet_kv:
+                continue
+            field_name, value = facet_kv.split(':', 1)
+
+            # Validate query facets as they as passed unescaped to Solr
+            if field_name in VALID_FACET_QUERIES:
+                if value not in VALID_FACET_QUERIES[field_name]:
+                    # Invalid query value
+                    continue
+
+            selected_multi_facets[field_name].append(value)
+
+        return selected_multi_facets
 
     def search(self):
         # We replace the 'search' method from FacetedSearchForm, so that we can
         # handle range queries
-
         # Note, we call super on a parent class as the default faceted view
         # escapes everything (which doesn't work for price range queries)
         sqs = super(FacetedSearchForm, self).search()
 
         # We need to process each facet to ensure that the field name and the
         # value are quoted correctly and separately:
-        for facet in self.selected_facets:
-            if ":" not in facet:
+        for field, values in self.selected_multi_facets.items():
+            if not values:
                 continue
-            field, value = facet.split(":", 1)
-            if value:
-                if field == 'price_exact':
-                    # Don't wrap value in speech marks and don't clean value
-                    sqs = sqs.narrow(u'%s:%s' % (field, value))
-                else:
-                    sqs = sqs.narrow(u'%s:"%s"' % (
-                        field, sqs.query.clean(value)))
+            if field in VALID_FACET_QUERIES:
+                # Query facet - don't wrap value in speech marks and don't
+                # clean value. Query values should have been validated by this
+                # point and so we don't need to escape them.
+                sqs = sqs.narrow(u'%s:(%s)' % (
+                    field, " OR ".join(values)))
+            else:
+                # Field facet - clean and quote the values
+                clean_values = [
+                    '"%s"' % sqs.query.clean(val) for val in values]
+                sqs = sqs.narrow(u'%s:(%s)' % (
+                    field, " OR ".join(clean_values)))
 
         if self.is_valid() and 'sort_by' in self.cleaned_data:
             sort_field = self.SORT_BY_MAP.get(
@@ -90,3 +136,13 @@ class SearchForm(FacetedSearchForm):
                 sqs = sqs.order_by(sort_field)
 
         return sqs
+
+
+class BrowseCategoryForm(SearchForm):
+    """
+    Variant of SearchForm that returns all products (instead of none) if no
+    query is specified.
+    """
+
+    def no_query_found(self):
+        return self.searchqueryset
