@@ -1,7 +1,3 @@
-import six
-
-from django.conf.urls import patterns
-
 from oscar.core.loading import feature_hidden
 from oscar.views.decorators import permissions_required
 
@@ -28,14 +24,14 @@ class Application(object):
     def __init__(self, app_name=None, **kwargs):
         self.app_name = app_name
         # Set all kwargs as object attributes
-        for key, value in six.iteritems(kwargs):
+        for key, value in kwargs.items():
             setattr(self, key, value)
 
     def get_urls(self):
         """
         Return the url patterns for this app.
         """
-        return patterns('')
+        return []
 
     def post_process_urls(self, urlpatterns):
         """
@@ -54,7 +50,7 @@ class Application(object):
         # Test if this the URLs in the Application instance should be
         # available.  If the feature is hidden then we don't include the URLs.
         if feature_hidden(self.hidable_feature_name):
-            return patterns('')
+            return []
 
         for pattern in urlpatterns:
             if hasattr(pattern, 'url_patterns'):
