@@ -13,17 +13,31 @@
 # copies of the Software, and to permit persons to whom the
 # Software is furnished to do so, subject to the following
 # conditions:
+#
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+# OTHER DEALINGS IN THE SOFTWARE.
 
-import six
+from django.utils import six
 
 from django.core import validators
 from django.conf import settings
 from django.core.exceptions import ValidationError
 from django.utils.translation import ugettext_lazy as _
+from django.utils.encoding import python_2_unicode_compatible
 
 import phonenumbers
 
 
+@python_2_unicode_compatible
 class PhoneNumber(phonenumbers.phonenumber.PhoneNumber):
     """
     A extended version of phonenumbers.phonenumber.PhoneNumber that provides
@@ -46,7 +60,7 @@ class PhoneNumber(phonenumbers.phonenumber.PhoneNumber):
                            keep_raw_input=True, numobj=phone_number_obj)
         return phone_number_obj
 
-    def __unicode__(self):
+    def __str__(self):
         format_string = getattr(
             settings, 'PHONENUMBER_DEFAULT_FORMAT', 'INTERNATIONAL')
         fmt = self.format_map[format_string]
@@ -54,9 +68,6 @@ class PhoneNumber(phonenumbers.phonenumber.PhoneNumber):
             return self.format_as(fmt)
         return self.raw_input
 
-    def __str__(self):
-        return str(self.__unicode__())
-
     def is_valid(self):
         """
         checks whether the number supplied is actually valid
@@ -86,7 +97,7 @@ class PhoneNumber(phonenumbers.phonenumber.PhoneNumber):
         return self.format_as(phonenumbers.PhoneNumberFormat.RFC3966)
 
     def __len__(self):
-        return len(self.__unicode__())
+        return len(six.text_type(self))
 
     def __eq__(self, other):
         if type(other) == PhoneNumber:
