@@ -4,6 +4,7 @@ from optparse import make_option
 from django.core.management.base import BaseCommand, CommandError
 
 from oscar.core.loading import get_class
+
 Importer = get_class('catalogue.utils', 'Importer')
 
 logger = logging.getLogger('oscar.catalogue.import')
@@ -24,7 +25,7 @@ class Command(BaseCommand):
         if len(args) != 1:
             raise CommandError('Command requires a path to a single folder')
 
-        logger.info("Starting image import...")
+        logger.info("Starting image import")
         dirname = args[0]
         importer = Importer(logger, field=options.get('filename'))
         importer.handle(dirname)
